//====================================================
// Map.C
//
// The Nipples World Map (symbols and types)
//
// ZNibbles
// Vincent Mallet 1997, 1998
//====================================================

// $Id: Map.C,v 1.2 1998/11/17 08:36:37 vmallet Exp $

/* ZNibbles - a small multiplayer game
 * Copyright (C) 1997, 1998 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <iostream.h>

#include "Map.H"

void Map::make(int x, int y)
{
  _x = x;
  _y = y;
  
  _map = new char * [y];
  _maptype = new DLList<MapType> * [y];
  
  for (int i = 0; i < y; i++) {
    _map[i] = new char[x];
    _maptype[i] = new DLList<MapType> [x];
  }

  clear();
}

Map::~Map()
{
  if (_map || _maptype) {
    for (int i = 0; i < _y; i++) {
      if (_map) delete[] _map[i];
      if (_maptype) delete[] _maptype[i];
    }
    if (_map) delete[] _map;
    if (_maptype) delete[] _maptype;
  }
}


void Map::clear(int c = '.')
{
  for (int i = 0; i < _y; i++)
    for (int j = 0; j < _x; j++) {
      _map[i][j] = c;
      for (Pix p = _maptype[i][j].first(); p; _maptype[i][j].del(p))
	;
    }
}


void Map::draw_number(int x, int y, int val)
{
  if (_map[y][x] == '.')
    _map[y][x] = val;
}

void Map::draw_point(int x, int y, int ch)
{
  _map[y][x] = ch;
}


void Map::display()
{
  cout << endl;
  
  for (int i = 0; i < _y; i++) {
    for (int j = 0; j < _x; j++)
      cout << _map[i][j] << " ";
    cout << endl;
  }
}

void Map::display_t()
{
  cout << endl;
  
  for (int i = 0; i < _y; i++) {
    for (int j = 0; j < _x; j++)
      cout << _maptype[i][j].length() << " ";
    cout << endl;
  }
}



void Map::add_type(int x, int y, _Object& obj)
{
  _maptype[y][x].append(&obj);
}


TypeList& Map::get_type(int x, int y)
{
  return _maptype[y][x];
}
